import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;


public class Client extends UnicastRemoteObject implements Facturation{
	String name;
	double monCompte = 10000; // initialement le client est riche !
	protected Client() throws RemoteException {
		super();
	}
	public Client(String name) throws RemoteException{
		super();
		this.name=name;
	}

	public static void main(String[] args){
		Client c=null;
		try {
			c = new Client(args[0]);
		} catch (RemoteException e1) {
			e1.printStackTrace();
		}
		System.out.println("Recherche d'un objet serveur");
	
/*		if (System.getSecurityManager() == null) { 
			System.setSecurityManager(new java.rmi.RMISecurityManager()); 
		}
		On peut remplacer ces lignes en crivant -Djava.security.manager 
		et pour debugger -Djava.security.debug=policy */
//		System.setProperty("java.security.policy","../java_policy");
		ConnexionServeur cs=null; // Cs est un stub vers l objet remote, renseign par le lookup
		try {
			cs = (ConnexionServeur)Naming.lookup("rmi://localhost" +
					":2004/Serveur"); 
				// rechercher sur cette machine, localhost, un objet remote offrant un service de connexion 
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		} catch (NotBoundException e) {
			e.printStackTrace();
		}
		Service s=null;
		try {
			s=cs.logon();
			System.out.println("On a recu une ref vers service distant "+s );
			c.travailler(s); // utiliser le service s pour raliser un travail
			System.out.println("Client a fini son boulot"); // mais on est bloqu car client est unicastRemoteObject!
		}catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}


	public  void travailler(Service s){ // precaution  prendre est de ne pas mettre cette mthode synchronized
		SmartProxyClient sp=new SmartProxyClient(this); // TOUT SE JOUE ICI pour le SMART PROXY
		// En effet, on instancie ce smart proxy, et on le passera comme rfrence sur this, mais, une rfrence "smart", 
		// puisque elle inclut dj le rsultat d'un futur appel  getName()
		try{
			for (int i=1;i<4;i++){
				int quant=2+i;
				int res=s.setVal(quant,sp);
				// On comprend bien qu'il ne faut pas envoyer la rfrence remote tel quel (this), car, cela engendrera forcment des appels remotes
				// A la place, on envoie en paramtre un proxy (sp) qui a un attribut pour contenir cette interface remote (pour les appels qu'il faudra forcment continuer  faire en remote
				// mais qui aura le rsultat des appels qu'on n'est pas obligs de faire en remote (en quelque sorte, le resultat est en 'cache' dans le proxy)
				// Bien sur, celui qui recoit ce smart proxy en paramtre ne "doit y voir que du feu"
				// Cad il se sert du smart proxy comme si c'tait un proxy remote habituel. Voir classe Facturation
				Thread.sleep(1000);
				System.out.println("Thread "+Thread.currentThread().getName() +" Valeur rendue par le service "+res);
			}		
		}catch (Exception e) {
			e.printStackTrace();
		}
	}
	// Cette mthode sera invoquee par l objet serveur pour savoir  quel nom tablir la facture
	// Dans un deuxieme exercice, on enverra un smart proxy de Client afin que le serveur ait dj
	// l information rendue par getName sans avoir besoin de faire un appel distant
	@Override
	public String getName()  {
		System.out.println("Thread="+Thread.currentThread().getName()+" is calling getName()");
		return name;
	}
	// Cette mthode sera invoquee par l objet serveur pour prsenter la facture au client
	@Override
	// facturer est synchronized car si le client a appel plusieurs services, ceux ci
	// doivent pouvoir se faire payer sans etre en concurrence (modif de la valeur monCompte)
	// Cela ne provoque pas interblocage malgr que  setVal soit aussi synchronized!
	// Un interblocage se produirait si travailler etait aussi synchronized, 
	// car cela empecherait le serveur d'invoquer facturer tant que le client serait
	// dans le moniteur car encore dans la mthode travailler
	public synchronized void  facturer(Facture f) throws RemoteException {
		monCompte-=f.montantfact;
		System.out.println("On vient de payer " + f.montantfact + " pour un service de "+ f.v + ", adressee  "+f.destinataire);
		// f.destination aura t rcupr par le serveur sans meme avoir eu besoin d invoquer getName via le reseau (info sera pose dans le smartproxy)
}

}